@@ -17,7 +17,7 @@
 /* The number of commits in the linear history from the
  * start of the universe up to the latest llvm main commit
  * that has been merged */
-#define LLVM_MAIN_REVISION 524190
+#define LLVM_MAIN_REVISION 554785
 
 /* Define if LLVM_ENABLE_DUMP is enabled */
 /* #undef LLVM_ENABLE_DUMP */
@@ -59,81 +59,6 @@
 /* LLVM name for the native target MCA init function, if available */
 /* #undef LLVM_NATIVE_TARGETMCA */
 
-/* Define if the AArch64 target is built in */
-#define LLVM_HAS_AARCH64_TARGET 0
-
-/* Define if the AMDGPU target is built in */
-#define LLVM_HAS_AMDGPU_TARGET 1
-
-/* Define if the ARC target is built in */
-#define LLVM_HAS_ARC_TARGET 0
-
-/* Define if the ARM target is built in */
-#define LLVM_HAS_ARM_TARGET 0
-
-/* Define if the AVR target is built in */
-#define LLVM_HAS_AVR_TARGET 0
-
-/* Define if the BPF target is built in */
-#define LLVM_HAS_BPF_TARGET 0
-
-/* Define if the CSKY target is built in */
-#define LLVM_HAS_CSKY_TARGET 0
-
-/* Define if the DirectX target is built in */
-#define LLVM_HAS_DIRECTX_TARGET 0
-
-/* Define if the Hexagon target is built in */
-#define LLVM_HAS_HEXAGON_TARGET 0
-
-/* Define if the Lanai target is built in */
-#define LLVM_HAS_LANAI_TARGET 0
-
-/* Define if the LoongArch target is built in */
-#define LLVM_HAS_LOONGARCH_TARGET 0
-
-/* Define if the M68k target is built in */
-#define LLVM_HAS_M68K_TARGET 0
-
-/* Define if the Mips target is built in */
-#define LLVM_HAS_MIPS_TARGET 0
-
-/* Define if the MSP430 target is built in */
-#define LLVM_HAS_MSP430_TARGET 0
-
-/* Define if the NVPTX target is built in */
-#define LLVM_HAS_NVPTX_TARGET 1
-
-/* Define if the PowerPC target is built in */
-#define LLVM_HAS_POWERPC_TARGET 0
-
-/* Define if the RISCV target is built in */
-#define LLVM_HAS_RISCV_TARGET 0
-
-/* Define if the Sparc target is built in */
-#define LLVM_HAS_SPARC_TARGET 0
-
-/* Define if the SPIRV target is built in */
-#define LLVM_HAS_SPIRV_TARGET 0
-
-/* Define if the SystemZ target is built in */
-#define LLVM_HAS_SYSTEMZ_TARGET 0
-
-/* Define if the VE target is built in */
-#define LLVM_HAS_VE_TARGET 0
-
-/* Define if the WebAssembly target is built in */
-#define LLVM_HAS_WEBASSEMBLY_TARGET 0
-
-/* Define if the X86 target is built in */
-#define LLVM_HAS_X86_TARGET 1
-
-/* Define if the XCore target is built in */
-#define LLVM_HAS_XCORE_TARGET 0
-
-/* Define if the Xtensa target is built in */
-#define LLVM_HAS_XTENSA_TARGET 0
-
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
 
@@ -147,7 +72,7 @@
 #define LLVM_USE_PERF 0
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 20
+#define LLVM_VERSION_MAJOR 22
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 0
@@ -156,7 +81,7 @@
 #define LLVM_VERSION_PATCH 0
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "20.0.0git"
+#define LLVM_VERSION_STRING "22.0.0git"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
@@ -181,6 +106,9 @@
 /* Define if LLVM is using tflite */
 /* #undef LLVM_HAVE_TFLITE */
 
+/* Define if we want to check profile consistency in lit tests */
+/* #undef LLVM_ENABLE_PROFCHECK */
+
 /* Define to 1 if you have the <sysexits.h> header file. */
 #define HAVE_SYSEXITS_H 1
 
@@ -190,6 +118,12 @@
 /* Define if building LLVM with BUILD_SHARED_LIBS */
 /* #undef LLVM_BUILD_SHARED_LIBS */
 
+/* Define if exporting LLVM public interface for shared library */
+#define LLVM_ENABLE_LLVM_EXPORT_ANNOTATIONS
+
+/* Define if exporting LLVM-C public interface for shared library */
+#define LLVM_ENABLE_LLVM_C_EXPORT_ANNOTATIONS
+
 /* Define if building LLVM with LLVM_FORCE_USE_OLD_TOOLCHAIN_LIBS */
 /* #undef LLVM_FORCE_USE_OLD_TOOLCHAIN */
 
@@ -206,4 +140,18 @@
 /* Define if logf128 is available */
 /* #undef LLVM_HAS_LOGF128 */
 
+/* Define if building LLVM with LLVM_ENABLE_TELEMETRY */
+#define LLVM_ENABLE_TELEMETRY 1
+
+/* Define to 1 to enable expensive checks for debug location coverage checking,
+   and to 0 otherwise. */
+#define LLVM_ENABLE_DEBUGLOC_TRACKING_COVERAGE 0
+
+/* Define to 1 to enable expensive tracking of the origin of debug location
+   coverage bugs, and to 0 otherwise. */
+#define LLVM_ENABLE_DEBUGLOC_TRACKING_ORIGIN 0
+
+/* Define to 1 to enable LLVM OnDisk Content Addressable Storage */
+#define LLVM_ENABLE_ONDISK_CAS 1
+
 #endif
